function answer = angles_in_polytope(poly,angles);
% function answer = angles_in_polytope(poly,angles);
% This program looks at the combinitorial structure of the polyhedron <poly>
% and the specified angles to see if the angles are in the polytope specified 
% by Andreev's theorem.
%
% RR 9/7/02


v = poly.faces;
a = poly.adjacency;
vert = poly.vert;

verbose = 0;

answer = 1;  %by default, the angles are in the polytope, unless one of the
	     %conditions 1-4 below is not satisfied.

faces = size(a,1);
vertices = size(vert,1);

%condition #1, all angles are non-obtuse

for i=1:faces;
  for j=i+1:faces;
    if (angles(i,j) > 90) 
      answer = 0;  %set to false
      if (verbose == 1)
        disp('Dihedral angle ');
        disp(i);
        disp(j);
        disp(' is greater than 90 degrees.');
      end;
    end
  end
end

for i=1:faces;
  for j=i+1:faces;
    if (a(i,j) == 1)
      for k=j+1:faces;
        if (a(j,k) == 1 & a(i,k) == 1)
          %otherwise this is a three circuit, either prismatic, or surrounding
          %a single vertex, i.e. a triangle.
          if (is_a_triangle(i,j,k,vert) == 1)
            if (angles(i,j) + angles(j,k) + angles(i,k) <= 180)
              answer = 0;
	      if (verbose == 1)
                vertex = [i j k]
                angles(i,j) + angles(j,k) + angles(i,k)
	      end
            end
          elseif (a(i,j) == 1 & a(i,k) == 1 & a(j,k) ==1);  
            %this is a prismatic 3-circuit!!!
            if (angles(i,j) + angles(j,k) + angles(i,k) >= 180)
              answer = 0;
	      if (verbose == 1)
	        prism = [i j k]
	        angles(i,j) + angles(j,k) + angles(i,k)
	      end;
            end;
	  end;
	elseif(a(j,k) == 1)
	  for l=k+1:faces
	    if(a(k,l) == 1 & a(i,l) == 1)
	      if(angles(i,j) + angles(j,k) + angles(k,l) + angles(i,l) >= 360)
	        answer = 0;
		if (verbose == 1)
	          [i j k l]
		end
              end
	    end
          end
	elseif(a(i,k) == 1)
	  for l=k+1:faces
	    if(a(l,j) == 1 & a(k,l) == 1)
	      if(angles(i,j)+angles(j,l)+angles(k,l)+angles(i,k) >= 360)
	        answer = 0;
		if (verbose == 1)
	          [i j k l]
		end
              end
	    end
	  end
        end;
      end;
    end;
  end
end



