function [v,a,vert,isom] = center(v_init,a_init,vert_init,isom_init)
%  function [v,a,vert,isom] = center(v_init,a_init,vert_init,isom_init)
%  This program centers the average of the vertices of a polyhedron at the
%  origin of the hyperbolic ball model.  Used as a helper function in
%  write_poly.m
%
%  RR 9/8/02


a = a_init;
vert = vert_init;
isom = isom_init;

[temp_v,temp_a,temp_vert] = isometry2inverse(v_init,a_init,vert_init,isom_init);

faces = size(a,1);
Faces = vert;  %this matrix contains in the i-th row, the list of 3 faces
                %that intersect at face i.
clear('S')
vertices = size(vert,1);



for i=1:vertices
  A(1,:) = temp_v(Faces(i,1),:);
  A(2,:) = temp_v(Faces(i,2),:);
  A(3,:) = temp_v(Faces(i,3),:);       %the vertex is orthogonal to each of the  A(:,1) = -A(:,1);
  vect = null(A);          %three planes
  norm = sqrt(-(inner(vect,vect)));
  vect = vect/norm;
  inner(vect,vect);
  if vect(1) < 0
    vect = -vect;
  end
  Vertices(i,:) = vect';
end

Vertices;
cent = center_poly(Vertices,vertices);

%determine the isometry of H3 taking the average of the vertices to (-1 0 0 0).
%this consists of doing a hyperbolic version of Graham-Schmidt on null spaces.

temp(1,:) = cent;
N = null(temp);
v1 = N(:,1)
v1 = v1/sqrt(inner(v1,v1));
temp(1,:) = cent;
temp(2,1) = -v1(1);
temp(2,2:4) = v1(2:4)';
N = null(temp);
v2 = N(:,1);
v2 = v2/sqrt(inner(v2,v2));
temp(3,1) = -v2(1); 
temp(3,2:4) = v2(2:4)';
v3 = null(temp);
v3 = v3/sqrt(inner(v3,v3));

A(1,1) = -cent(1);
A(1,2:4) = cent(2:4);
A(2,:) = v1';
A(3,:) = v2';
A(4,:) = v3';

New_Vertices = (A*Vertices')';
center_poly(New_Vertices,vertices);

%now we must convert the vertices back to faces.

for n=1:vertices
  r = 2./(1-Vertices(n,2)^2-Vertices(n,3)^2-Vertices(n,4)^2);
  Vertices(n,:) = r*Vertices(n,:);
  Vertices(n,1) = sqrt(1+Vertices(n,2)^2+Vertices(n,3)^2+Vertices(n,4)^2);
end



%now we need to use the faces array to compute the vectors in the dual
%hyperboloid that describe the faces.



center_of_poly = [-1 0 0 0];

for m=1:faces
  v_on_face = vertices_on_face(m,vert)
  D(1,:)=Vertices(v_on_face(1),:);
  D(2,:)=Vertices(v_on_face(2),:);
  D(3,:)=Vertices(v_on_face(3),:);

  D(:,1)=-D(:,1);

  temp = null(D);
  norm = sqrt(abs(inner(temp,temp)));
  temp = temp/norm;
  %center_of_poly = [1 0 0 0];
  if (inner(center_of_poly,temp) < 0)  %make sure that the half-space
    temp = -temp;                       %specified by v contains the
  end                                   %inside of the polyhedron.
  Vectors(m,:) = temp';
end

temp_v = Vectors;

[v,a,vert] = isometry2inverse(temp_v,temp_a,temp_vert,isom^-1);








%__________________________________________________________________________
%this is a helper function

function l = center_poly(V,vertices)
sum = zeros(1,4);
for i=1:vertices
  sum = sum + V(i,:);
end
norm = sqrt(abs(inner(sum,sum)));
l = sum/norm;


function v_on_face = vertices_on_face(face,vert);
vertices = size(vert,1);
count = 1;
for m=1:vertices
  for n = 1:3
    if (vert(m,n) == face)
      v_on_face(count) = m;
      count = count +1;
    end
  end
end



%__________________________________________________________________________

