function  poly_out = center_my_poly(poly)
% function  write_poly(poly,file_name)
% Converts internal format to geomview *.off format for hyperbolic polyhedra.
% 
% WARNING: YOUR FILENAME MUST END IN .off FOR GEOMVIEW TO READ IT.
%
% RR 8/2/02


Vectors = poly.faces;
adjacency = poly.adjacency;
vert = poly.vert;

num_faces = size(adjacency,1);
Faces = vert;  %this matrix contains in the i-th row, the list of 3 faces
		%that intersect at face i.
clear('S')
vertices = size(vert,1);



for i=1:vertices
  A(1,:) = Vectors(Faces(i,1),:);
  A(2,:) = Vectors(Faces(i,2),:);
  A(3,:) = Vectors(Faces(i,3),:);	%the vertex is orthogonal to each of the
  A(:,1) = -A(:,1);
  v = null(A);		%three planes
  norm = sqrt(-(inner(v,v)));
  v = v/norm;
  inner(v,v);
  if v(1) < 0
    v = -v;
  end
   %V = v(2:4)/(v(1)+1); %conformal coords
   %V = v(2:4)/(v(1)); %projective coords--geomview takes projective coors.
   Vertices(i,:) = v';
end

Vertices;

%__________________________________________________________________________

cent = center_poly(Vertices,vertices);

%determine the isometry of H3 taking the average of the vertices to (-1 0 0 0).
%this consists of doing a hyperbolic version of Graham-Schmidt on null spaces.

temp(1,:) = cent;
N = null(temp);
v1 = N(:,1);
v1 = v1/sqrt(inner(v1,v1));
temp(1,:) = cent;
temp(2,1) = -v1(1);
temp(2,2:4) = v1(2:4)';
N = null(temp);
v2 = N(:,1);
v2 = v2/sqrt(inner(v2,v2));
temp(3,1) = -v2(1);
temp(3,2:4) = v2(2:4)';
v3 = null(temp);
v3 = v3/sqrt(inner(v3,v3));

A(1,1) = -cent(1);
A(1,2:4) = cent(2:4);
A(2,:) = v1';
A(3,:) = v2';
A(4,:) = v3';

A


poly_out.vert = poly.vert;
poly_out.adjacency = poly.adjacency;
poly_out.faces = zeros(num_faces,4)

for i=1:num_faces;
  poly_out.faces(i,:) = (A*poly.faces(i,:)')';
end







