function [poly_out,k,l] = comb_whitehead(i,j,poly_in)
% function [poly_out,k,l] = comb_whitehead(i,j,poly_in)
% This program takes a polyhedron and perfomes a whitehead move on an edje (i,j)
% where i and j label faces
%
% RR 11/16/02


poly_out.faces = poly_in.faces;
a_init = poly_in.adjacency;
vert_init = poly_in.vert;

%dimensions
faces = size(a_init,1);
vertices = size(vert_init,1);

if (a_init(i,j) ~= 1)
  i
  j
  disp('There is no edje (i,j), try again!');
  pause
  poly_out = poly_in;
  %break
  return
end

%first determine the other faces k and l intersect with faces i and j.
init = 0;
for m = 1:vertices
  count = 0;
  for n = 1:3
    if(vert_init(m,n) == i | vert_init(m,n) == j)
      count = count +1;
    else
      temp = vert_init(m,n);
    end
  end
  if (count == 2 & init == 0)
    k = temp;
    init = 1;
    first = m;
  elseif (count == 2 & init == 1)
    l = temp;
    second = m;
    break;
  end
end

%modify vert to make it appropriate.
vert_out = vert_init;
vert_out(first,1) = i;
vert_out(first,2) = k;
vert_out(first,3) = l;
vert_out(second,1) = j;
vert_out(second,2) = k;
vert_out(second,3) = l;

poly_out.vert = vert_out;

a_out = a_init;
a_out(i,j) = 0;		%a_out needs to represent the new combinatorics.
a_out(j,i) = 0;
a_out(k,l) = 1;
a_out(l,k) = 1;
poly_out.adjacency = a_out;

if (is_3prismatic(poly_out) == 1)
  disp('Going 3-prismatic!!  WARNING!')
  pause
  pause
  pause
end

