function [poly_new,whitehead,inv_whitehead] = cond_cycle(poly,M,num_cycle,cycle,num_interior,interior,v_inf)
%  function [poly_new,whitehead,inv_whitehead] = condition_interior_to_cycle(poly,M,num_cycle,cycle,num_interior,interior,v_inf)
% Given, the interior->cycle matrix M, the cycle, the interior, and v_inf
% this program determines the necessary whitehead moves to increase the number
% on edges in the interior polygon by 1.
%
% This is the new technology, courtesy of ME!!!
%
% RR 2/11/03
% last modified 5/18/03


%first, determine the number of components of parts of the cycle which connect
%to any specific interior vertex:

poly_temp = poly;  % so that you don't mess up the initial poly.
whitehead = [0 0];  % just to cue up whitehead, later will be fixed.
inv_whitehead = [0 0];  % just to cue up inv_whitehead, later will be fixed.
num_moves = 0;

for i=1:num_interior;
  for j = 1:num_interior;
    M_int(i,j) = poly.adjacency(interior(i),interior(j));
  end
end

%M_int

%Now we use this to classify the interior points.  Those that are connected
%to only one are refered to as 'endpoints', those connected to exactly two
%interior points are known as 'linearpoints' and those connected to more
%than two interior points are known as 'junctionpoints'.

for i=1:num_interior
  int_class(i) = norm(M_int(i,:),1) - 1;  %subtract one, don't count self-adjacency
  num_connected(i) = norm(M(:,i),1);
end

%int_class
%num_connected

N = zeros(num_interior,1);
for i=1:num_interior
  j = 1;
  N(i) = 0;
  while (j <= num_cycle)
    if (j == 1)
      if (M(1,i) == 1)
        N(i) = N(i) + 1;
      end
    else
      if (M(j,i) == 1 & M(j-1,i) == 0)
        N(i) = N(i) + 1;
      end
    end
    j = j+1;
  end
  if (M(1,i) == 1 & M(num_cycle,i) == 1)
    N(i) = N(i) - 1;
  end
end

%POSSIBLY A WAY TO SPEED THIS UP WOULD BE TO ELIMINATE THE COMPUTATION OF
%N, SINCE IT IS ONLY USED TO COMPUTE THE FOLLOWING BELOW.  A FASTER ALGORITHM
%WOULD INTEGRATE THE TWO.

max_component = zeros(num_interior,2);
%Will store the component haveing the most connections and how many
%points is is connected to.
for i=1:num_interior
  k = 1;
  for j = 1:N(i)
    while(M(k,i) == 0)
      k = k+1;
    end
    temp_k = k;
    temp_num = 0;
    while(k <= num_cycle & M(k,i) == 1)
      k = k+1;
      temp_num = temp_num+1;
    end
    if (temp_num > max_component(i,2))
      max_component(i,2) = temp_num;
      max_component(i,1) = temp_k;
    end
  end
  %The following fancy-ness is to take care of the case where the biggest component
  %contains the first point in the cycle.  We must count any connections at 
  %num_cycle, and work backwards adding any extra's via the "wrap around effect."
  if (k < num_cycle & max_component(i,1) == 1)
    k = num_cycle;
    while(M(k,i) == 1)
      max_component(i,1) = k;
      k = k-1;
      max_component(i,2) = max_component(i,2) + 1;
    end
  end
end

%max_component

%Jesus, I hope that works correctly!!!!!!!!!!!  I think that it does.
      
      
      

%Now we go through the cases of the proof:

%CASE 2: there is an interior point which is not an endpoint and is 
%connected to a component having more than one vertex in it.  In terms
%of our variables, that means int_class(i) > 1, max_component(i,2) > 2.

%We try to find the one that has the minimum num_connected(i)

interior_preferred = 0;
num = num_cycle;
for i=1:num_interior;
  %search for an interior vertex which has a max component connected to 2 or more.
  if ((int_class(i) > 1) & (max_component(i,2) > 1))
    if (num_connected(i) < num)
      num = num_connected(i)
      interior_preferred = i;
    end
  end
end

if (interior_preferred ~=0 )
  whichcase = 2;
else
  end_point = 0;
  end_point_preferred = 0; 
  for i=1:num_interior;
    %search for an interior vertex which is an endpoint and is connected to more than 3
    if ((int_class(i) == 1) & (max_component(i,2) > 3))
      end_point_preferred = i;
    end
    if (int_class(i) == 1)
      end_point = i;
    end
  end
  if (end_point_preferred > 0)
    whichcase = 1;
  else
    whichcase = 3;
  end
end




%_________________________________________________________________________
% CASE 1.
if (whichcase == 1)  
  disp('Case 1')
  %end_point_preferred
  outer1_index = max_component(end_point_preferred,1);
  outer2_index = fix_index(max_component(end_point_preferred,1) + max_component(end_point_preferred,2)-1,num_cycle);
  outer1 = cycle(outer1_index);
  outer2 = cycle(outer2_index);
  inner_index = 0;
  for i=1:num_interior;
    if ((M_int(i,end_point_preferred) == 1) & (i ~= end_point_preferred))
      inner_index = i;
    end
  end
  inner = interior(inner_index);

  %Now, we "borrow" a connection to the outer cycle from the endpoint preferred
  %so that inner is connected to one component having 2 connections and
  %the other component having only one connection.
  [poly_temp,k,l] = comb_whitehead(outer1,interior(end_point_preferred),poly_temp);
  num_moves = num_moves + 1;
  whitehead(num_moves,1) = outer1;
  whitehead(num_moves,2) = interior(end_point_preferred);
  inv_whitehead(num_moves,1) = k;
  inv_whitehead(num_moves,2) = l;
  M = interior_to_cycle(poly_temp,num_cycle,cycle,num_interior,interior);

%  %Now, we eliminate the component in which inner is connected to only
%  %one point on the cycle.
%  [poly_temp,k,l] = comb_whitehead(outer2,inner,poly_temp);
%  num_moves = num_moves + 1;
%  whitehead(num_moves,1) = outer2;
%  whitehead(num_moves,2) = inner;
%  inv_whitehead(num_moves,1) = k;
%  inv_whitehead(num_moves,2) = l;
%  M = interior_to_cycle(poly_temp,num_cycle,cycle,num_interior,interior)

  i = fix_index(outer1_index+2,num_cycle)
  while (i ~= outer1_index)
    if(M(i,inner_index) == 1)
      [poly_temp,k,l] = comb_whitehead(cycle(i),inner,poly_temp);
      num_moves = num_moves + 1;
      whitehead(num_moves,1) = cycle(i);
      whitehead(num_moves,2) = inner;
      inv_whitehead(num_moves,1) = k;
      inv_whitehead(num_moves,2) = l;
    end
    i = fix_index(i+1,num_cycle);
  end
  M = interior_to_cycle(poly_temp,num_cycle,cycle,num_interior,interior);


  % At this point, the interior_preferred column of M should have one

  %Now, we increase the length of the cycle by doing the Whitehead move
  %on the edge formed by the two consecutive parts of the cycle
  %that inner is connected to.
  cycle1 = outer1;
  cycle2 = cycle(fix_index(outer1_index+1,num_cycle));
  %outer1_index
  mod(outer1_index,num_cycle)+1
  [poly_temp,k,l] = comb_whitehead(cycle1,cycle2,poly_temp);
  num_moves = num_moves + 1;
  whitehead(num_moves,1) = cycle1;
  whitehead(num_moves,2) = cycle2;
  inv_whitehead(num_moves,1) = k;
  inv_whitehead(num_moves,2) = l;

  poly_new = poly_temp;


   

%_________________________________________________________________________
% CASE 2.

elseif (whichcase == 2)
  disp('Case 2')
  interior_preferred
  %num
  %First, we must take the other components where interior_preferred is
  %connected to the outer cycle and do Whitehead moves to eliminate these
  %connections.
  i_start = fix_index(max_component(interior_preferred,1) + max_component(interior_preferred,2),num_cycle)
  disp('Max')
  i_end = max_component(interior_preferred,1)
  i = i_start;
  while (i ~= i_end)
    if(M(i,interior_preferred) == 1)
      [poly_temp,k,l] = comb_whitehead(cycle(i),interior(interior_preferred),poly_temp);
      num_moves = num_moves + 1;
      whitehead(num_moves,1) = cycle(i);
      whitehead(num_moves,2) = interior(interior_preferred);
      inv_whitehead(num_moves,1) = k;
      inv_whitehead(num_moves,2) = l;
    end
    i = fix_index(i+1,num_cycle);
  end
  % At this point, the interior_preferred column of M should have one
  % connected component having two or more 1's in it.
  M = interior_to_cycle(poly_temp,num_cycle,cycle,num_interior,interior);

  num_remove = max_component(interior_preferred,2) -2;
  i = 0;
  j = max_component(interior_preferred,1)
  while (i < num_remove)
     disp(j+i)
     [poly_temp,k,l] = comb_whitehead(cycle(fix_index(j+i,num_cycle)),interior(interior_preferred),poly_temp);
     num_moves = num_moves + 1;
     whitehead(num_moves,1) = cycle(fix_index(j+i,num_cycle));
     whitehead(num_moves,2) = interior(interior_preferred);
     inv_whitehead(num_moves,1) = k;
     inv_whitehead(num_moves,2) = l;
     i = i+1;
  end
  M = interior_to_cycle(poly_temp,num_cycle,cycle,num_interior,interior);

  %Now the only connection should be at j+i and j+i+1.  Do the Whitehead move
  %on this edge to increase num_cycle.
   [poly_temp,k,l] = comb_whitehead(cycle(fix_index(j+i,num_cycle)),cycle(fix_index(j+i+1,num_cycle)),poly_temp);
   num_moves = num_moves + 1;
   whitehead(num_moves,1) = cycle(fix_index(j+i,num_cycle));
   whitehead(num_moves,2) = cycle(fix_index(j+i+1,num_cycle));
   inv_whitehead(num_moves,1) = k;
   inv_whitehead(num_moves,2) = l;

   poly_new = poly_temp;



%_________________________________________________________________________
% CASE 3.
else
  disp('Case 3')
  num_line = 1;
  inner_index = 0;
  line_index(num_line) = end_point;
  line(num_line) = interior(end_point);
  while(int_class(line_index(num_line)) < 3)
    for i = 1:num_interior
      if ((M_int(i,line_index(num_line)) == 1) & (i ~= line_index(num_line)))
        if (num_line == 1  | i ~= line_index(num_line -1))
          num_line = num_line+1;
          line_index(num_line) = i;
          line(num_line) = interior(i);
        end
      end
    end
  end
  %line_index

  outer1_index = max_component(end_point,1);
  outer2_index = mod(max_component(end_point,1) + max_component(end_point,2)-1,num_cycle+1);
  outer1 = cycle(outer1_index);
  outer2 = cycle(outer2_index);

  %We must eliminate any connections between line(num_line) and cycle, other
  %than outer1 and outer2.
  for i=1:num_cycle
    if (M(i,line_index(num_line)) == 1 & i ~= outer1_index  & i ~= outer2_index)
      [poly_temp,k,l] = comb_whitehead(line(num_line),cycle(i),poly_temp);
      num_moves = num_moves + 1;
      whitehead(num_moves,1) = line(num_line);
      whitehead(num_moves,2) = cycle(i);
      inv_whitehead(num_moves,1) = k;
      inv_whitehead(num_moves,2) = l;
    end
  end
  M = interior_to_cycle(poly_temp,num_cycle,cycle,num_interior,interior);
      

  %Do the whithead move between line(num_line) and outer2 making it so that
  %line(num_line) is attached to cycle in only one component, having only
  %one point in it.
  [poly_temp,k,l] = comb_whitehead(line(num_line),outer2,poly_temp);
  num_moves = num_moves + 1;
  whitehead(num_moves,1) = line(num_line);
  whitehead(num_moves,2) = outer2;
  inv_whitehead(num_moves,1) = k;
  inv_whitehead(num_moves,2) = l;
  M = interior_to_cycle(poly_temp,num_cycle,cycle,num_interior,interior);
 

  %The following moves will open up the connection between line(num_line)
  %and cycle, so that line(num_line) is attached to two points on cycle.
  i = num_line-1;
  while(i > 0)
     [poly_temp,k,l] = comb_whitehead(line(i),outer1,poly_temp);
     num_moves = num_moves + 1;
     whitehead(num_moves,1) = line(i);
     whitehead(num_moves,2) = outer1;
     inv_whitehead(num_moves,1) = k;
     inv_whitehead(num_moves,2) = l;
     i = i - 1;
  end
  M = interior_to_cycle(poly_temp,num_cycle,cycle,num_interior,interior);

  other_index = mod(outer1_index,num_cycle)+1; 

  %Now, the point line(num_line) is connected to ther outer cycle
  %in exactly 2 points, outer1 and cycle(other_index).  We do the WH
  %move on this edge to increase the length of the cycle by 1.
  [poly_temp,k,l] = comb_whitehead(outer1,cycle(other_index),poly_temp);
  num_moves = num_moves + 1;
  whitehead(num_moves,1) = outer1;
  whitehead(num_moves,2) = cycle(other_index);
  inv_whitehead(num_moves,1) = k;
  inv_whitehead(num_moves,2) = l;

  poly_new = poly_temp;

end

  
 

