function poly = mod_prism(n)
%  function poly = mod_prism(n)
%  creates a modified prism with n faces.




num_vertices = 2*(n-2);
num_faces = n;
sides = n-3;

for i=1:sides
  verticies(i,2) = .95*cos(i*2*pi/sides);
  verticies(i,3) = .95*sin(i*2*pi/sides);
  verticies(i,4) = -.1;
end
  
for i=sides+1:2*sides
  verticies(i,2) = .95*cos(i*2*pi/sides);
  verticies(i,3) = .95*sin(i*2*pi/sides);
  verticies(i,4) = .1;
end

%  These are the ones that split the first face of the prism.
verticies(2*sides+1,2) = .95*cos(2*pi/sides);
verticies(2*sides+1,3) = .95*sin(2*pi/sides);
verticies(2*sides+1,4) = 0;

verticies(2*sides+2,2) = .98*cos(4*pi/sides);
verticies(2*sides+2,3) = .98*sin(4*pi/sides);
verticies(2*sides+2,4) = 0;


V = verticies

faces(1,1) = sides;
faces(2,1) = sides;
for i=1:sides;
  faces(1,i+1) = i-1;
  faces(2,i+1) = sides+i-1;
end

for i=1:sides-2;
    faces(i+2,1) = 4;
    faces(i+2,2) = i;
    faces(i+2,3) = i+1;
    faces(i+2,4) = sides+i+1;
    faces(i+2,5) = sides+i;
end

faces(sides+1,1) = 4;
faces(sides+1,2) = sides-1;
faces(sides+1,3) = 2*sides-1;
faces(sides+1,4) = sides;
faces(sides+1,5) = 0;

%here are the two special faces, one on top of the other.
faces(sides+2,1) = 4;
faces(sides+2,2) = 0;
faces(sides+2,3) = 1;
faces(sides+2,4) = 2*sides+1;
faces(sides+2,5) = 2*sides;

faces(sides+3,1) = 4;
faces(sides+3,2) = sides;
faces(sides+3,3) = sides+1;
faces(sides+3,4) = 2*sides+1;
faces(sides+3,5) = 2*sides;

faces


%We have now entered the data in geomview format.  The rest
%is the conversion to internal format.


for i=0:num_vertices-1;
  n = 1;
  for j =1:num_faces
    for k=2:faces(j,1)+1
      if (faces(j,k) == i)
        vertices_spec(i+1,n) = j;
        n = n+1;
      end
    end
  end
end

%convert the vertices to 4-vectors on the hyperboloid:

for n=1:num_vertices
  r_squared = (V(n,2)^2+V(n,3)^2+V(n,4)^2);
  V(n,:) = V(n,:)/(1-r_squared);
  V(n,1) = sqrt(1+V(n,2)^2+V(n,3)^2+V(n,4)^2);
end

V;
temp = V(1:3,:);
temp(:,1) = -temp(:,1);
null(temp);



%now we need to use the faces array to compute the vectors in the dual
%hyperboloid that describe the faces.



center_of_poly = center_poly(V,num_vertices);

for m=1:num_faces
  D(1,:)=V(faces(m,2)+1,:);
  D(2,:)=V(faces(m,3)+1,:);
  D(3,:)=V(faces(m,4)+1,:);

  D(:,1)=-D(:,1);

  temp = null(D);
  norm = sqrt(abs(inner(temp,temp)));
  temp = temp/norm;
  %center_of_poly = [1 0 0 0];
  if (inner(center_of_poly,temp) < 0)  %make sure that the half-space
    temp = -temp;                       %specified by v contains the
  end                                   %inside of the polyhedron.
  Vectors(m,:) = temp';
end

%Compute the adjacency matrix:

faces;

adjacency = zeros(num_faces,num_faces);

for f1 = 1:num_faces
  for f2 = f1:num_faces

    n = 0;
    for i = 1:faces(f1,1)
      for j = 1:faces(f2,1)
        if (faces(f1,i+1) == faces(f2,j+1))
          n = n + 1;
        end
      end
    end
    if (n >= 2)
      adjacency(f1,f2) = 1;
      adjacency(f2,f1) = 1;
    end

  end
end

poly.faces = Vectors;
poly.adjacency = adjacency;
poly.vert = vertices_spec;


