function poly_out = whitehead(i,j,poly_in)
% function poly_out = whitehead(i,j,poly_in)
% This program takes a polyhedron and perfomes a whitehead move on an edje (i,j)
% where i and j label faces
%
% RR 9/5/02


v_init = poly_in.faces;
a_init = poly_in.adjacency;
vert_init = poly_in.vert;

%dimensions
faces = size(a_init,1);
vertices = size(vert_init,1);

center = zeros(1,4);
for n=1:faces;
  center = center + v_init(n,:);
end

for n=1:faces;
  if (inner(v_init(n,:),center) > 0)
    v_init(n,:) = -v_init(n,:);
  end
end;

poly_in.faces = v_init;

  


if (a_init(i,j) ~= 1)
  disp('There is no edje (i,j), try again!');
  pause
  poly_out = poly_in;
  return
end

%first determine the other faces k and l intersect with faces i and j.
init = 0;
for m = 1:vertices
  count = 0;
  for n = 1:3
    if(vert_init(m,n) == i | vert_init(m,n) == j)
      count = count +1;
    else
      temp = vert_init(m,n);
    end
  end
  if (count == 2 & init == 0)
    k = temp;
    init = 1;
    first = m;
  elseif (count == 2 & init == 1)
    l = temp;
    second = m;
    break;
  end
end

%k
%l
   
   


%modify vert to make it appropriate.
vert_out = vert_init;
vert_out(first,1) = i;
vert_out(first,2) = k;
vert_out(first,3) = l;
vert_out(second,1) = j;
vert_out(second,2) = k;
vert_out(second,3) = l;

   
angles = compute_angles(poly_in);
angles(i,k) = 90.0;
angles(k,i) = 90.0;
angles(i,l) = 90.0;
angles(l,i) = 90.0;
angles(j,k) = 90.0;
angles(k,j) = 90.0;
angles(j,l) = 90.0;
angles(l,j) = 90.0;
angles(i,j) = 5;
angles(j,i) = 5;

n =  1;
for m = 1:5;
  count = 0;
  for p = 1:3; 
    if(vert_init(m,p) == i | vert_init(m,p) == j)
      count = count +1;
    end
  end
  if (count ~= 2)
    n = m;
  end
end

%n 

disp('Preparing to do Whitehead move.');
%angles_in = angles_in_polytope(poly_in,angles);
poly_temp1 = newton_iter2(poly_in,angles,n,1e-5,100);
%angles

write_poly(poly_temp1,'pre_wh.off')


%compute_angles(poly_temp1)

%this is the temporary polyhedron that will be perturbed to modify the
%combinatorics.  Notice that it is fine to have newton's method 'hold on to'
%any vertex, say vertex 1, since we aren't changing the combinatorics.

a_out = a_init;
a_out(i,j) = 0;		%a_out needs to represent the new combinatorics.
a_out(j,i) = 0;
a_out(k,l) = 1;
a_out(l,k) = 1;
poly_out.adjacency = a_out;
poly_temp1.adjacency = a_out;


%solve for the polyhedron with the faces slightly shifted, so that the 
%Whitehead move has been achieved.
angles(i,j) = -10;
angles(j,i) = -10;
angles(k,l) = 5;
angles(l,k) = 5;

%angles

%we will need to 'hold on to' a vertex that isn't near the move:

disp('Doing Whitehead move. ************************************************');
poly_temp2 = newton_iter2(poly_temp1,angles,n,1e-6,1);

disp('Putting in canonical positon');
poly_temp2.adjacency = a_out;
poly_temp2.vert = vert_out;
write_poly(poly_temp2,'post_wh.off')
poly_out = poly_temp2;
poly_out.adjacency = a_out;
poly_out.vert = vert_out;
poly_out = canonical_choose_vertex(poly_out,n,100);

    




