%this program takes a polyhedron and perfomes a whitehead move on an edje (i,j)
% where i and j label faces

% RR 9/5/02

function poly_out = whitehead(i,j,poly_in)

v_init = poly_in.faces;
a_init = poly_in.adjacency;
vert_init = poly_in.vert;

%dimensions
faces = size(a_init,1);
vertices = size(vert_init,1);


if (a_init(i,j) ~= 1)
  disp('There is no edje (i,j), try again!');
  v = v_init;
  a = a_init;
  vert = vert_init;
  pause
  break
end

%first determine the other faces k and l intersect with faces i and j.
%c = 0;
%for n = 1:faces
%  if (n ~= i & n ~=j)
%    if(a_init(n,i) == 1 & a_init(n,j) == 1 & c == 0);
%      c = 1;
%      k = n;
%    elseif (a_init(n,i) == 1 & a_init(n,j) & c == 1);
%      l = n;
%      break;  %you have found both faces that intersect with faces i and j.
%    end
%  end
%end

%first determine the other faces k and l intersect with faces i and j.
init = 0;
for m = 1:vertices
  count = 0;
  for n = 1:3
    if(vert_init(m,n) == i | vert_init(m,n) == j)
      count = count +1;
    else
      temp = vert_init(m,n);
    end
  end
  if (count == 2 & init == 0)
    k = temp;
    init = 1;
    first = m;
  elseif (count == 2 & init == 1)
    l = temp;
    second = m;
    break;
  end
end
   
   


%modify vert to make it appropriate.
vert = vert_init;
vert(first,1) = i;
vert(first,2) = k;
vert(first,3) = l;
vert(second,1) = j;
vert(second,2) = k;
vert(second,3) = l;
   

angles = compute_angles(poly_in);
angles(i,k) = 89;
angles(k,i) = 89;
angles(i,l) = 89;
angles(l,i) = 89;
angles(j,k) = 89;
angles(k,j) = 89;
angles(j,l) = 89;
angles(l,j) = 89;
angles(i,j) = 3;
angles(j,i) = 3;


disp('Preparing to do Whithead move.');
angles_in_polytope(poly_in,angles)
temp = newton_iter(poly_in,angles,1,1e-10);

v_temp = temp.faces;

%this is the temporary polyhedron that will be perturbed to modify the
%combinatorics

a = a_init;
a(i,j) = 0;		%a needs to represent the new combinatorics.
a(j,i) = 0;
a(k,l) = 1;
a(l,k) = 1;

   


%solve for the polyhedron with the faces slightly shifted, so that the 
%Whitehead move has been achieved.
angles(i,j) = -10;
angles(j,i) = -10;
angles(k,l) = 3;
angles(l,k) = 3;
angles(1,j+1) = 72;
angles(1,j-1) = 30;
angles(j-2,j-1) = 88;
angles(1,j-2) = 88;

disp('Doing Whithead move.');

temp_poly.faces = v_temp;
temp_poly.adjacency = a;
temp_poly.vert = vert;
angles_in_polytope(temp_poly,angles)
v_modified = newton_iter(v_temp,a,vert,angles,1e-10);


angles(k,l) = 72;
angles(l,k) = 72;
%v_modified = newton_iter(v_modified,a,vert,angles,1e-10);

v = v_modified;


poly_out.faces = v;
poly_out.adjacency = a;
poly_out.vert = vert;
    




