function  write_poly(poly,file_name)
% function  write_poly(poly,file_name)
% Converts internal format to geomview *.off format for hyperbolic polyhedra.
% 
% WARNING: YOUR FILENAME MUST END IN .off FOR GEOMVIEW TO READ IT.
%
% RR 8/2/02


Vectors = poly.faces;
adjacency = poly.adjacency;
vert = poly.vert;

faces = size(adjacency,1);
Faces = vert;  %this matrix contains in the i-th row, the list of 3 faces
		%that intersect at face i.
clear('S')
vertices = size(vert,1);



for i=1:vertices
  A(1,:) = Vectors(Faces(i,1),:);
  A(2,:) = Vectors(Faces(i,2),:);
  A(3,:) = Vectors(Faces(i,3),:);	%the vertex is orthogonal to each of the
  A(:,1) = -A(:,1);
  v = null(A);		%three planes
  norm = sqrt(-(inner(v,v)));
  v = v/norm;
  inner(v,v);
  if v(1) < 0
    v = -v;
  end
   %V = v(2:4)/(v(1)+1); %conformal coords
   %V = v(2:4)/(v(1)); %projective coords--geomview takes projective coors.
   Vertices(i,:) = v';
end

Vertices;

%__________________________________________________________________________

cent = center_poly(Vertices,vertices);

%determine the isometry of H3 taking the average of the vertices to (-1 0 0 0).
%this consists of doing a hyperbolic version of Graham-Schmidt on null spaces.

temp(1,:) = cent;
N = null(temp);
v1 = N(:,1);
v1 = v1/sqrt(inner(v1,v1));
temp(1,:) = cent;
temp(2,1) = -v1(1);
temp(2,2:4) = v1(2:4)';
N = null(temp);
v2 = N(:,1);
v2 = v2/sqrt(inner(v2,v2));
temp(3,1) = -v2(1);
temp(3,2:4) = v2(2:4)';
v3 = null(temp);
v3 = v3/sqrt(inner(v3,v3));

A(1,1) = -cent(1);
A(1,2:4) = cent(2:4);
A(2,:) = v1';
A(3,:) = v2';
A(4,:) = v3';

%Put in center of mass position
Vertices = (A*Vertices')';
center_poly(Vertices,vertices);


%_________________________________________________________________________

for i=1:vertices
   v = Vertices(i,:);
   V = v(2:4)/(v(1)); %projective coords--geomview takes projective coors.
   New_Vertices(i,:) = V;
end




vertex_adj = zeros(vertices,vertices);

for i=1:vertices;
  for j=1:vertices;
    if (share_two_faces(i,j,Faces))
      vertex_adj(i,j) = 1;
    end
  end
end

%output the number of verticies and faces, then the actual verticies
%in the Poincare model.
fid = fopen(file_name,'w');
fprintf(fid,'%d %d %d\n',vertices,faces,faces+vertices-2);
for i=1:vertices
  fprintf(fid,'\n');
  fprintf(fid,'%f ',New_Vertices(i,:));
end
fprintf(fid,'\n');


for n = 1:faces;
  vertices_per_face = 1;
  clear('F')

  i = 1;
  while (vertex_contained_in_face(i,n,Faces) ~= 1)
    i = i+1;
  end
  p1 = i;
  F(1) = p1-1;
  current_vertex = p1;
  prev_vertex = p1;

  i = 1;
  while (share_two_faces(i,current_vertex,Faces)==0 | (i==current_vertex) | (i == prev_vertex) | (vertex_contained_in_face(i,n,Faces)~=1))
    i = i+1;
  end
  current_vertex = i;
  vertices_per_face = 2;
  F(2) = i-1;

  while (current_vertex ~= p1)
       i = 1;
       while (share_two_faces(i,current_vertex,Faces)==0 | (i==current_vertex) | (i == prev_vertex) | (vertex_contained_in_face(i,n,Faces)~=1))
         i = i+1;
       end
       prev_vertex = current_vertex;
       current_vertex = i;
       vertices_per_face = vertices_per_face + 1;
       F(vertices_per_face) = i-1;
  end
  F(vertices_per_face+1) = n;
  F(1) = vertices_per_face-1;
  %M(n,1) = vertices_per_face-1;
  %M(n,2:vertices_per_face) = F(1:vertices_per_face-1);
  %M(n,vertices_per_face+1) = n;  %assign it the n+1st color in the geomview
  fprintf(fid,'\n');
  fprintf(fid,'%d ',F);
  
end				 








