function write_poly_snappea(poly,filename)
% function write_poly_snappea(poly,filename)
% Writes a list of generators (as elements of SO(3,1)) of the reflection
% group in the faces of poly in a format suitable for use by SnapPea.
% Note: to be able to load this file into SnapPea, the polyhedron
% poly must have all dihedral angles pi/n_i, where n_i is an integer
% greater than or equal to 2 that is assigned to edge e_i.

Vectors = poly.faces;
adjacency = poly.adjacency;
vert = poly.vert;

fid = fopen(filename,'w');

fprintf(fid,'%% Generators \n');

%output the number of generators
%in the Poincare model.
num_faces = size(adjacency,1);
fprintf(fid,'%d\n\n',num_faces);

for i=1:num_faces
  v = Vectors(i,:);
  A = compute_reflection(v);
  det(A)
  fprintf(fid,'%26.16f  ',A(1,:));
  fprintf(fid,'\n');
  fprintf(fid,'%26.16f  ',A(2,:));
  fprintf(fid,'\n');
  fprintf(fid,'%26.16f  ',A(3,:));
  fprintf(fid,'\n');
  fprintf(fid,'%26.16f  ',A(4,:));
  fprintf(fid,'\n\n');
end

fclose(fid)
